<?php
/*
Plugin Name: Batch image upload
Plugin URI: http://mogrify.homelinux.org/
Description: Upload and resize a bunch of images at once
Version: 0.1
Author: Tony Gambone
Author URI: http://mogrify.homelinux.org/
*/

/* Actions */
add_action('admin_head','bi_show_head');
add_action('admin_menu','bi_add_admin_page');
add_action('init','bi_process_form');

/**
 * Admin page
 */
function bi_add_admin_page() {
  add_submenu_page(
    'post.php',     		// parent page
    'Batch image upload',	// page title
    'Upload images',		// name in menu
    'edit_posts',			// user capability
    null,					// content file 
    'bi_show_form'		// content function
  );
}

/**
 * Display the upload form.
 */
function bi_show_form() {
  global $title;
  global $home;
  global $bi_debug;
  global $bi_files;

  if (!extension_loaded('gd')) { bi_show_gd_error(); return; }

  // set up options
  $options = get_option('bi_options');

  // default values
  if (false === $options) {
    $options = array(
      'do_resize' => 1,
      'max_height' => 400,
      'max_width' => 400,
      'do_debug' => 0,
      'num_controls' => 10
    );
  }
  foreach ($options as $name => $opt) { $$name = $opt; }

  // HTML setup for options
  $html_resize = ($do_resize) ? ' checked="checked"' : "";
  $html_debug  = ($do_debug) ? ' checked="checked"' : "";

  // Debugging info
  $display_results = (@is_array($bi_files) && $bi_files !== array());
  if ($display_results) {
    $debug = ($do_debug) ? "<div class=\"bi_box\"><h3>Debugging info:</h3>\n" . $bi_debug . "<pre>".@print_r($bi_files,true)."</pre></div>" : "";
  }
  
  // Generate HTML for uploaded images
  $img_html = "";
  if ($display_results) {
    $img_html = "<h3>Your images were
    uploaded successfully.</h3>
    <p>To use them in a post,
    copy and paste the code below.</p>\n<div>";
    foreach ($bi_files as $file) {
      $img_html .= htmlentities(
        sprintf('<img src="%s" alt="%s"/>', 
          preg_replace('@^https?://.*?/@','/',$file['url']), 
          basename($file['url'])
        )
        ) . "<br/>";
    }
    $img_html = "<div class=\"bi_box\">\n$img_html\n</div>\n</div>\n";
  }

print <<<OUT
<div class="wrap bi">
  <form enctype="multipart/form-data" method="post">
    <div>
      <div id="moremeta">
        <div id="grabit" class="dbx-group">
          <fieldset class="dbx-box">
            <h3 class="dbx-handle">Options</h3>
            <div class="dbx-content">
              <label class="selectit">
                <input name="bi_do_resize" id="bi_do_resize" value="1" type="checkbox"$html_resize/>
                Limit image size
              </label>
              <label for="bi_max_width" class="selectit">
                <input name="bi_max_width" id="bi_max_width" value="$max_width" type="text" class="bi_max"/>
                Max width
              <label for="bi_max_height" class="selectit">
                <input name="bi_max_height" id="bi_max_height" value="$max_height" type="text" class="bi_max"/>
                Max height
              </label>
              <hr/>
              <label for="bi_num_controls" class="selectit">
                Show 
                <input name="bi_num_controls" id="bi_num_controls" value="$num_controls" type="text" class="bi_max"/>
                initial upload slots
              </label>
              <hr/>
              <label class="selectit">
                <input name="bi_do_debug" id="bi_do_debug" value="1" type="checkbox"$html_debug/>
                Show debugging info
              </label>
            </div>
          </fieldset>
        </div>
      </div>

      <h2 id="write-post">$title</h2>
      <p>Uploaded files must be GIF, JPEG, or PNG images.  Other image and file
      types will be discarded.</p>

      $img_html
      $debug
      <br style="clear: left;"/>

      <input type="hidden" name="action" value="save"/>
      <input type="hidden" name="bi_process_form" value="true"/>
      <input type="button" value="Add another slot" onclick="bi_add_control()" class="bi_submit"/>
      <div id="bi_ctlcontainer">
OUT;
for ($i=0;$i<$num_controls;$i++) {
print <<<CTL
      <label id="bi_filectl">Image:
        <input type="file" name="bi_files[]" size="50"/>
        <br/>
      </label>
CTL;
}
print <<<OUT
      </div>
      <input type="submit" value="Upload/Save options" class="bi_submit"/>
      <input type="button" value="Add another slot" onclick="bi_add_control()" class="bi_submit"/>
    </div>
  </form>
</div>
OUT;
}

/**
 * Handle form submission.
 */
function bi_process_form() {
  $debugmsg = "";

  if ($_SERVER['REQUEST_METHOD'] == 'POST' && $_POST['bi_process_form']) {
    if (!extension_loaded('gd')) return;
    require_once(ABSPATH . '/wp-admin/admin-functions.php');

    // get options
    $do_resize = (@$_POST['bi_do_resize']) ? 1 : 0;
    $max_width = intval($_POST['bi_max_width']);
    $max_height = intval($_POST['bi_max_height']);
    $do_debug = (@$_POST['bi_do_debug']) ? 1 : 0;
    $num_controls = intval($_POST['bi_num_controls']);
  
    // force sane values
    $max_width = ($max_width < 1) ? 1 : $max_width;
    $max_height = ($max_height < 1) ? 1 : $max_height;
    // this must be at least 2 because of PHP's $_FILES structure
    $num_controls = ($num_controls < 2) ? 2 : $num_controls;

    // save options
    $debugmsg = "Saving option choices.<br/>";
    update_option('bi_options', 
      array(
        'do_resize' => $do_resize, 
        'max_width' => $max_width, 
        'max_height' => $max_height, 
        'do_debug' => $do_debug,
        'num_controls' => $num_controls
    ));
 
    $debugmsg .= "Processing files.<br/>";

    $bi_files = array(); // place to store uploaded file info

    // Resize each image and pass it to Wordpress' upload handling function.
    for ($i=0; $i < count($_FILES['bi_files']['name']); $i++) {
      $debugmsg .= "New file (" . $_FILES['bi_files']['name'][$i] . ").<br/>";

      // array of file info
      $file = array(
        'name' 		=> &$_FILES['bi_files']['name'][$i],
        'type' 		=> &$_FILES['bi_files']['type'][$i],
        'tmp_name' 	=> &$_FILES['bi_files']['tmp_name'][$i],
        'error'		=> &$_FILES['bi_files']['error'][$i],
        'size' 		=> &$_FILES['bi_files']['size'][$i]
      );

      // make sure it's a supported image
      if (($imginfo = @getimagesize($file['tmp_name'])) === FALSE) {
        $debugmsg .= "Not an image.<br/>";
        continue;
      }
      if ($imginfo[2] != IMAGETYPE_GIF && 
          $imginfo[2] != IMAGETYPE_JPEG && 
          $imginfo[2] != IMAGETYPE_PNG) {
        $debugmsg .= "Not a supported image.<br/>";
        continue;
      }

      $debugmsg .= "It's a supported image.<br/>";

      // resize the image (if it needs to be)
      list($w_orig,$h_orig,$type,$html) = $imginfo;
      if ($do_resize && 
          !($w_orig <= $max_width && $h_orig <= $max_height)) {
        $debugmsg .= "Resizing.<br/>";

        // calculate dimensions
        $r_orig = $w_orig / $h_orig;
        if ($max_width/$max_height > $r_orig) {
          $w_new = $max_height * $r_orig;
          $h_new = $max_height;
        } else {
          $w_new = $max_width;
          $h_new = $max_width / $r_orig;
        }

        // determine gd create function to use from filetype
        switch($type) {
          case IMAGETYPE_GIF:
            $img_orig = imagecreatefromgif($file['tmp_name']);
            break;
          case IMAGETYPE_JPEG:
            $img_orig = imagecreatefromjpeg($file['tmp_name']);
            break;
          case IMAGETYPE_PNG:
            $img_orig = imagecreatefrompng($file['tmp_name']);
            break;
        }

        // resize
        $img_new = imagecreatetruecolor($w_new,$h_new);
        imagecopyresampled(
          $img_new, $img_orig,
          0, 0, 0, 0,
          $w_new, $h_new,
          $w_orig, $h_orig
        );

        // determine gd save function to use from filetype
        switch($type) {
          case IMAGETYPE_GIF:
            imagegif($img_new, $file['tmp_name']);
            break;
          case IMAGETYPE_JPEG:
            imagejpeg($img_new, $file['tmp_name']);
            break;
          case IMAGETYPE_PNG:
            imagepng($img_new, $file['tmp_name']);
            break;
        }       
        $debugmsg .= "The file was resized.<br/>";
      } else {
        $debugmsg .= "Resizing was skipped.<br/>";
      }

      // this will be an associative array with keys
      // file, url, type, or just error if something went wrong
      $bi_files[] = wp_handle_upload( $file, array('action'=>'save') );
      $debugmsg .= "The upload was was processed.<br/>";
    }    
    $GLOBALS['bi_files'] = $bi_files;
  }
  $GLOBALS['bi_debug'] = $debugmsg;
}

/**
 * What to show when GD isn't available.
 */
function bi_show_gd_error() {
print <<<ERROR
<div class="wrap">
  <h2 id="write-post">$title</h2>
  <p>This isn't done yet. Don't use it.</p>
  <p>Your PHP does not have support for image manipulation. Please
  enable the <a href="http://us2.php.net/gd">gd</a> extension to use
  this plugin.</p>
</div>
ERROR;
}

/**
 * Things to add to the page header.
 */
function bi_show_head() {
print <<<HEAD
<style type="text/css">
  .bi hr { text-align: left; width: 90%; }
  .bi .selectit { margin-bottom: 2px; }
  .bi_max { width: 30px; }
  .bi_submit { margin: 10px 6px; }
  .bi_box {
    border: 1px solid #999;
    background-color: #f6f6f6;
    margin: 6px;
    padding: 6px;
    color: #333;
    float: left;
    clear: both;
  }
  .bi_box h3 {
    margin-top: 0px;
    font-size: 100%;
    font-weight: bold;
  }  
</style>
<script type="text/javascript">
function bi_add_control() {
  var ctl = document.getElementById('bi_filectl');
  var container = document.getElementById('bi_ctlcontainer');
  if (!ctl || !container) alert('Not supported on this page.');
  var newNode = ctl.cloneNode(true);  
  container.appendChild(newNode);
}
</script>
HEAD;
}

/*  Copyright 2006  Tony Gambone  (email : tonygambone@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

?>
